==> apache-eventmesh-1.4.0-incubating-source.tar.gz: apache-eventmesh-1.4.0-incubating-src/eventmesh-runtime/src/main/java/org/apache/eventmesh/runtime/core/protocol/tcp/client/group/dispatch/FreePriorityDispatchStrategy.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.eventmesh.runtime.core.protocol.tcp.client.group.dispatch;

import org.apache.eventmesh.runtime.core.protocol.tcp.client.session.Session;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class FreePriorityDispatchStrategy implements DownstreamDispatchStrategy {

    private static final Logger logger = LoggerFactory.getLogger(FreePriorityDispatchStrategy.class);

    @Override
    public Session select(String group, String topic, Set<Session> groupConsumerSessions) {
        if (CollectionUtils.isEmpty(groupConsumerSessions)
                || StringUtils.isBlank(topic)
                || StringUtils.isBlank(group)) {
            return null;
        }

        List<Session> filtered = new ArrayList<Session>();
        List<Session> isolatedSessions = new ArrayList<>();
        for (Session session : groupConsumerSessions) {
            if (!session.isAvailable(topic)) {
                continue;
            }
            if (session.isIsolated()) {
                isolatedSessions.add(session);
                logger.info("session is not available because session is isolated,isolateTime:{},client:{}",
                        session.getIsolateTime(), session.getClient());
                continue;
            }
            filtered.add(session);
        }

        if (CollectionUtils.isEmpty(filtered)) {
            if (CollectionUtils.isEmpty(isolatedSessions)) {
                logger.warn("all sessions can't downstream msg");
                return null;
            } else {
                logger.warn("all sessions are isolated,group:{},topic:{}", group, topic);
                filtered.addAll(isolatedSessions);
            }
        }

        Collections.shuffle(filtered);
        Session session = filtered.get(0);
        return session;
    }
}

==> apache-eventmesh-1.8.0-incubating-source.tar.gz: apache-eventmesh-1.8.0-incubating-src/eventmesh-runtime/src/main/java/org/apache/eventmesh/runtime/core/protocol/tcp/client/group/dispatch/FreePriorityDispatchStrategy.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.eventmesh.runtime.core.protocol.tcp.client.group.dispatch;

import org.apache.eventmesh.runtime.core.protocol.tcp.client.session.Session;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class FreePriorityDispatchStrategy implements DownstreamDispatchStrategy {

    private static final Logger LOGGER = LoggerFactory.getLogger(FreePriorityDispatchStrategy.class);

    @Override
    public Session select(final String group, final String topic, final Set<Session> groupConsumerSessions) {
        if (CollectionUtils.isEmpty(groupConsumerSessions)
                || StringUtils.isBlank(topic)
                || StringUtils.isBlank(group)) {
            return null;
        }

        final List<Session> filtered = new ArrayList<>();
        final List<Session> isolatedSessions = new ArrayList<>();
        for (final Session session : groupConsumerSessions) {
            if (!session.isAvailable(topic)) {
                continue;
            }

            if (session.isIsolated()) {
                isolatedSessions.add(session);
                if (LOGGER.isInfoEnabled()) {
                    LOGGER.info("session is not available because session is isolated,isolateTime:{},client:{}",
                            session.getIsolateTime(), session.getClient());
                }
                continue;
            }

            filtered.add(session);
        }

        if (CollectionUtils.isEmpty(filtered)) {
            if (CollectionUtils.isEmpty(isolatedSessions)) {
                if (LOGGER.isWarnEnabled()) {
                    LOGGER.warn("all sessions can't downstream msg");
                }
                return null;
            } else {
                if (LOGGER.isWarnEnabled()) {
                    LOGGER.warn("all sessions are isolated,group:{},topic:{}", group, topic);
                }
                filtered.addAll(isolatedSessions);
            }
        }

        Collections.shuffle(filtered);
        return filtered.get(0);
    }
}
